@@ -7,43 +7,46 @@ import warnings
 from abc import ABC, abstractmethod
 from itertools import groupby, zip_longest
 from pathlib import Path
-from typing import Any, Callable, Iterable, Iterator, Literal, Sequence, TypeVar, overload
+from typing import (Any, Callable, Iterable, Iterator, Literal, Sequence,
+                    TypeVar, overload)
 
-from vskernels import Catrom
-from vstools import (DependencyNotFoundError, Direction, FormatsMismatchError, InvalidColorFamilyError, Matrix,
-                     VariableFormatError, check_variable, check_variable_format, check_variable_resolution, core, depth,
-                     get_prop, get_subsampling, get_w)
+from vskernels import Catrom, Kernel, KernelT, Point
+from vstools import (CustomError, CustomNotImplementedError, CustomTypeError,
+                     CustomValueError, DependencyNotFoundError, Direction,
+                     FormatsMismatchError, InvalidColorFamilyError,
+                     LengthMismatchError, Matrix, Sentinel,
+                     UnsupportedSubsamplingError, VariableFormatError,
+                     check_variable, check_variable_format,
+                     check_variable_resolution, clip_async_render, core, depth,
+                     get_prop, get_subsampling, get_w, merge_clip_props)
 from vstools import split as split_planes
 from vstools import vs
 
-from .dehardsub import hardsub_mask
 from .exceptions import ClipsAndNamedClipsError
 from .misc import source
-from .render import clip_async_render
-from .types import x265_me_map
 from .util import truncate_string
 
 __all__ = [
     'compare', 'comp',
     'Comparer',
-    'diff_hardsub_mask',
+    'comparison_shots',
     'diff',
-    'Interleave',
-    'interleave',
+    'Interleave', 'interleave',
     'source_mediainfo', 'srcm',
-    'Split',
-    'split',
+    'Split', 'split',
     'stack_compare', 'scomp',
-    'stack_horizontal',
+    'stack_horizontal', 'stack_vertical',
     'stack_planes',
-    'stack_vertical',
     'Stack',
-    'Tile',
-    'tile',
+    'Tile', 'tile',
 ]
 
 T = TypeVar('T')
 
+x265_me_map = [
+    "dia", "hex", "umh", "star", "sea", "full"
+]
+
 
 class Comparer(ABC):
     """
@@ -72,11 +75,11 @@ class Comparer(ABC):
                  label_alignment: int = 7
                  ) -> None:
         if len(clips) < 2:
-            raise ValueError("Comparer: Compare functions must be used on at least 2 clips!")
+            raise CustomValueError("Compare functions must be used on at least 2 clips!", self.__class__)
         if label_alignment not in list(range(1, 10)):
-            raise ValueError("Comparer: `label_alignment` must be an integer from 1 to 9!")
+            raise CustomValueError("`label_alignment` must be an integer from 1 to 9!", self.__class__)
         if not isinstance(clips, (dict, Sequence)):
-            raise TypeError(f"Comparer: Unexpected type {type(clips)} for `clips` argument!")
+            raise CustomTypeError(f"Unexpected type {type(clips)} for `clips` argument!", self.__class__)
 
         self.clips = list(clips.values()) if isinstance(clips, dict) else list(clips)
         self.names = list(clips.keys()) if isinstance(clips, dict) else None
@@ -104,7 +107,7 @@ class Comparer(ABC):
 
     @abstractmethod
     def _compare(self) -> vs.VideoNode:
-        raise NotImplementedError
+        raise CustomNotImplementedError("This function or method has not been implemented yet!", self.__class__)
 
     @property
     def clip(self) -> vs.VideoNode:
@@ -152,11 +155,11 @@ class Stack(Comparer):
     def _compare(self) -> vs.VideoNode:
         if self.direction == Direction.HORIZONTAL:
             if not self.height:
-                raise ValueError("Stack: StackHorizontal requires that all clips be the same height!")
+                raise CustomValueError("StackHorizontal requires that all clips be the same height!", self.__class__)
             return core.std.StackHorizontal(self._marked_clips())
 
         if not self.width:
-            raise ValueError("Stack: StackVertical requires that all clips be the same width!")
+            raise CustomValueError("StackVertical requires that all clips be the same width!", self.__class__)
         return core.std.StackVertical(self._marked_clips())
 
 
@@ -255,14 +258,14 @@ class Tile(Comparer):
         super().__init__(clips, label_alignment=label_alignment)
 
         if not self.width or not self.height:
-            raise ValueError("Tile: All clip widths and heights must be the same!")
+            raise CustomValueError("All clip widths and heights must be the same!", self.__class__)
 
         if arrangement is None:
             self.arrangement = self._auto_arrangement()
         else:
             is_one_dim = len(arrangement) < 2 or all(len(row) == 1 for row in arrangement)
             if is_one_dim:
-                raise ValueError("Tile: Use Stack instead if the array is one dimensional!")
+                raise CustomValueError("Use Stack instead if the array is one dimensional!", self.__class__)
             self.arrangement = arrangement
 
         self.blank_clip = core.std.BlankClip(clip=self.clips[0], keep=1)
@@ -272,8 +275,8 @@ class Tile(Comparer):
 
         array_count = sum(map(sum, self.arrangement))  # type:ignore[arg-type]
 
-        if array_count != self.num_clips:
-            raise ValueError('Tile: Specified arrangement has an invalid number of clips!')
+        LengthMismatchError.check(self.__class__, array_count, self.num_clips,
+                                  message="Specified arrangement has an invalid number of clips!")
 
     def _compare(self) -> vs.VideoNode:
         clips = self._marked_clips()
@@ -330,7 +333,7 @@ class Split(Stack):
     def _smart_crop(self) -> None:  # has to alter self.clips to send clips to _marked_clips() in Stack's _compare()
         """Crops self.clips in place accounting for odd resolutions."""
         if not self.width or not self.height:
-            raise ValueError("Split: All clips must have same width and height!")
+            raise CustomValueError("All clips must have same width and height!", self.__class__)
 
         breaks_subsampling = ((self.direction == Direction.HORIZONTAL and (((self.width // self.num_clips) % 2)
                                                                            or ((self.width % self.num_clips) % 2)))
@@ -340,8 +343,8 @@ class Split(Stack):
         is_subsampled = not all(get_subsampling(clip) in ('444', None) for clip in self.clips)
 
         if breaks_subsampling and is_subsampled:
-            raise ValueError("Split: Resulting cropped width or height violates subsampling rules! "
-                             "Consider resampling to YUV444 or RGB before attempting to crop!")
+            raise CustomValueError("Resulting cropped width or height violates subsampling rules! "
+                                   "Consider resampling to YUV444 or RGB before attempting to crop!", self.__class__)
 
         match self.direction:
             case Direction.HORIZONTAL:
@@ -384,40 +387,41 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
 
     Alias for this function is ``lvsfunc.comp``.
 
-    :param clip_a:          Clip to compare.
-    :param clip_b:          Second clip to compare.
-    :param frames:          List of frames to compare (Default: ``None``).
-    :param rand_total:      Number of random frames to pick (Default: ``None``).
-    :param force_resample:  Forcibly resamples the clip to RGB24 (Default: ``True``).
-    :param print_frame:     Print frame numbers (Default: ``True``).
-    :param mismatch:        Allow for clips with different formats and dimensions to be compared (Default: ``False``).
+    :param clip_a:                      Clip to compare.
+    :param clip_b:                      Second clip to compare.
+    :param frames:                      List of frames to compare (Default: ``None``).
+    :param rand_total:                  Number of random frames to pick (Default: ``None``).
+    :param force_resample:              Forcibly resamples the clip to RGB24 (Default: ``True``).
+    :param print_frame:                 Print frame numbers (Default: ``True``).
+    :param mismatch:                    Allow for clips with different formats and dimensions
+                                        to be compared (Default: ``False``).
 
-    :return:                Interleaved clip containing specified frames from `clip_a` and `clip_b`.
+    :return:                            Interleaved clip containing specified frames from `clip_a` and `clip_b`.
 
-    :raises ValueError:     More comparisons requested than frames available.
-    :raises FormatsMismatchError:
-                            Format of given clips don't match.
+    :raises VariableResolutionError:    One of the given clips is of a variable resolution.
+    :raises ValueError:                 More comparisons requested than frames available.
+    :raises VariableFormatError:        `mismatch` is False and one of the given clips is of a variable resolution.
+    :raises FormatsMismatchError:       `mismatch` is False and format of given clips don't match.
     """
     def _resample(clip: vs.VideoNode) -> vs.VideoNode:
         # Resampling to 8 bit and RGB to properly display how it appears on your screen
-        return core.resize.Bicubic(clip, format=vs.RGB24, matrix_in=Matrix.from_video(clip),
-                                   dither_type='error_diffusion')
+        return Catrom.resample(clip, vs.RGB24, None, Matrix.from_video(clip), dither_type="error_diffusion")
 
-    check_variable_resolution(clip_a, "compare")
-    check_variable_resolution(clip_b, "compare")
+    check_variable_resolution(clip_a, compare)
+    check_variable_resolution(clip_b, compare)
 
     # Error handling
     if frames and len(frames) > clip_a.num_frames:
-        raise ValueError("compare: 'More comparisons requested than frames available!'")
+        raise CustomValueError("More comparisons requested than frames available!", compare,
+                               reason=f"{len(frames)} > {clip_a.num_frames}")
 
     if force_resample:
         clip_a, clip_b = _resample(clip_a), _resample(clip_b)
     elif mismatch is False:
-        assert check_variable_format(clip_a, "compare")
-        assert check_variable_format(clip_b, "compare")
+        assert check_variable_format(clip_a, compare)
+        assert check_variable_format(clip_b, compare)
 
-        if clip_a.format.id != clip_b.format.id:
-            raise FormatsMismatchError("compare")
+        FormatsMismatchError.check(compare, clip_a, clip_b)
 
     if print_frame:
         clip_a = clip_a.text.Text("Clip A").text.FrameNum(alignment=9)
@@ -453,7 +457,7 @@ def stack_compare(*clips: vs.VideoNode,
     :raises ValueError:     More or less than 2 clips are given.
     """
     if len(clips) != 2:
-        raise ValueError(f"stack_compare: You must pass at least two clips, not {len(clips)}.")
+        raise CustomValueError("You must pass at least two clips", stack_compare, f"{len(clips)} < 2")
 
     clipa, clipb = clips
 
@@ -487,8 +491,9 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
 
     :return:                            Clip with stacked planes.
 
+    :raises VariableFormatError:        Clip is of a variable format.
+    :raises VariableResolutionError:    Clip is of a variable resolution.
     :raises InvalidColorFamilyError:    Clip is not YUV or RGB.
-    :raises TypeError:                  Clip is of an unexpected color family.
     :raises TypeError:                  Clip returns an unexpected subsampling.
     """
     assert check_variable(clip, stack_planes)
@@ -501,8 +506,6 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
         planes: dict[str, vs.VideoNode] = {'Y': yuv_planes[0], 'U': yuv_planes[1], 'V': yuv_planes[2]}
     elif clip.format.color_family == vs.ColorFamily.RGB:
         planes = {'R': yuv_planes[0], 'G': yuv_planes[1], 'B': yuv_planes[2]}
-    else:
-        raise TypeError(f"stack_planes: Unexpected color family: {clip.format.color_family.name}!")
 
     direction: Direction = Direction.HORIZONTAL if not stack_vertical else Direction.VERTICAL
 
@@ -515,24 +518,7 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
 
         return Stack([y_plane, subsampled_planes], direction=direction).clip
     else:
-        raise TypeError(f"stack_planes: Unexpected subsampling {get_subsampling(clip)}!")
-
-
-def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
-    """
-    Diff func for :py:func:`lvsfunc.comparison.diff` to use a hardsub mask.
-
-    This is kinda slow.
-
-    :param a:   Clip A.
-    :param b:   Clip B.
-
-    :return:    Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`.
-    """
-    assert check_variable(a, "diff_hardsub_mask")
-    assert check_variable(b, "diff_hardsub_mask")
-
-    return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b, **kwargs))
+        raise UnsupportedSubsamplingError(stack_planes)
 
 
 @overload
@@ -541,8 +527,10 @@ def diff(*clips: vs.VideoNode,
          height: int = ...,
          interleave: bool = ...,
          return_ranges: Literal[True] = True,
+         avg_thr: float | Literal[True] = True,
          exclusion_ranges: Sequence[int | tuple[int, int]] | None = ...,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
+         msg: str = ...,
          **namedclips: vs.VideoNode) -> tuple[vs.VideoNode, list[tuple[int, int]]]:
     ...
 
@@ -553,8 +541,10 @@ def diff(*clips: vs.VideoNode,
          height: int = ...,
          interleave: bool = ...,
          return_ranges: Literal[False],
+         avg_thr: float | Literal[True] = True,
          exclusion_ranges: Sequence[int | tuple[int, int]] | None = ...,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
+         msg: str = ...,
          **namedclips: vs.VideoNode) -> vs.VideoNode:
     ...
 
@@ -564,8 +554,10 @@ def diff(*clips: vs.VideoNode,
          height: int = 288,
          interleave: bool = False,
          return_ranges: bool = False,
+         avg_thr: float | Literal[True] = True,
          exclusion_ranges: Sequence[int | tuple[int, int]] | None = None,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = lambda a, b: core.std.MakeDiff(a, b),
+         msg: str = "Diffing clips...",
          **namedclips: vs.VideoNode) -> vs.VideoNode | tuple[vs.VideoNode, list[tuple[int, int]]]:
     """
     Create a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
@@ -596,34 +588,40 @@ def diff(*clips: vs.VideoNode,
                                 (using :py:class:`lvsfunc.comparison.Interleave`).
                                 This will not return a diff clip
     :param return_ranges:       Return a list of ranges in addition to the comparison clip.
+    :param avg_thr:             Threshold of additional average diff check.
+                                True to automatically calculate based on thr.
     :param exclusion_ranges:    Excludes a list of frame ranges from difference checking output (but not processing).
     :param diff_func:           Function for calculating diff in PlaneStatsMin/Max mode.
+    :param msg:                 Message for the progress bar. Defaults to "Diffing clips...".
+                                Useful if you're using `diff` for some other process.
 
     :return:                    Either an interleaved clip of the differences between the two clips
                                 or a stack of both input clips on top of MakeDiff clip.
                                 Furthermore, the function will print the ranges of all the diffs found.
 
-    :raises ClipsAndNamedClipsError:
-                                Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError: Both positional and named clips are given.
     :raises ValueError:         More or less than two clips are given.
     :raises ValueError:         ``thr`` is 128 or higher.
-    :raises VariableFormatError:
-                                One of the clips is of a variable format.
+    :raises VariableFormatError: One of the clips is of a variable format.
+    :raises LengthMismatchError: The given clips are of different lengths.
     :raises StopIteration:      No differences are found.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("diff")
+        raise ClipsAndNamedClipsError(diff)
 
     if (clips and len(clips) != 2) or (namedclips and len(namedclips) != 2):
-        raise ValueError("diff: Must pass exactly 2 `clips` or `namedclips`!")
+        raise CustomValueError("Must pass exactly 2 `clips` or `namedclips`!", diff)
 
-    if thr >= 128:
-        raise ValueError("diff: `thr` must be below 128!")
+    if abs(thr) > 128:
+        raise CustomValueError("`thr` must be between [-128, 128]!", diff, thr)
+
+    if avg_thr is True:
+        avg_thr = max(0.012, (128 - thr) * 0.000046875 + 0.0105) if thr > 1 else 0.0
 
     if clips and not all([c.format for c in clips]):
-        raise VariableFormatError("diff")
+        raise VariableFormatError(diff)
     elif namedclips and not all([nc.format for nc in namedclips.values()]):  # noqa
-        raise VariableFormatError("diff")
+        raise VariableFormatError(diff)
 
     def _to_ranges(iterable: list[int]) -> Iterable[tuple[int, int]]:
         iterable = sorted(set(iterable))
@@ -637,28 +635,36 @@ def diff(*clips: vs.VideoNode,
         nc = list(namedclips.values())
         a, b = depth(nc[0], 8), depth(nc[1], 8)
 
-    frames = []
+    LengthMismatchError.check(diff, a.num_frames, b.num_frames)
+
+    callbacks = []
+
     if thr <= 1:
-        ps = core.std.PlaneStats(a, b)
+        callbacks.append(lambda f: get_prop(f, 'PlaneStatsDiff', float) > thr)
 
-        def _cb(n: int, f: vs.VideoFrame) -> None:
-            if get_prop(f, 'PlaneStatsDiff', float) > thr:
-                frames.append(n)
-
-        clip_async_render(ps, progress="Diffing clips...", callback=_cb)
-        diff = core.std.MakeDiff(a, b)
+        diff_clip = merge_clip_props(a.std.MakeDiff(b), a.std.PlaneStats(b))
     else:
-        diff = diff_func(a, b).std.PlaneStats()
-        t = float if diff.format.sample_type == vs.FLOAT else int  # type:ignore[union-attr]
+        diff_clip = diff_func(a, b).std.PlaneStats()
 
-        def _cb(n: int, f: vs.VideoFrame) -> None:
-            if get_prop(f, 'PlaneStatsMin', t) <= thr or get_prop(f, 'PlaneStatsMax', t) >= 255 - thr > thr:
-                frames.append(n)
+        typ = float if diff_clip.format and diff_clip.format.sample_type == vs.FLOAT else int
 
-        clip_async_render(diff, progress="Diffing clips...", callback=_cb)
+        callbacks.append(
+            lambda f: get_prop(f, 'PlaneStatsMin', typ) <= thr or get_prop(f, 'PlaneStatsMax', typ) >= 255 - thr > thr
+        )
+
+        if avg_thr > 0.0:
+            diff_clip = merge_clip_props(diff_clip, a.std.PlaneStats(b, None, 'PAVG'))
+
+            callbacks.append(lambda f: get_prop(f, 'PAVGDiff', float) > avg_thr)
+
+    frames_render = clip_async_render(
+        diff_clip, None, msg, lambda n, f: Sentinel.check(n, any(cb(f) for cb in callbacks))
+    )
+
+    frames = list(Sentinel.filter(frames_render))
 
     if not frames:
-        raise StopIteration("diff: No differences found!")
+        raise CustomError['StopIteration']('No differences found!', diff)  # type: ignore[index]
 
     frames.sort()
 
@@ -667,7 +673,7 @@ def diff(*clips: vs.VideoNode,
         for e in exclusion_ranges:
             if isinstance(e, int):
                 e = (e, e)
-            start, end = e[0], e[1]+1
+            start, end = e[0], e[1] + 1
             r += list(range(start, end))
         frames = [f for f in frames if f not in r]
 
@@ -682,15 +688,15 @@ def diff(*clips: vs.VideoNode,
                                  f'{name_b}': core.std.Splice([b[f] for f in frames])}).clip
     else:
         scaled_width = get_w(height, mod=1)
-        diff = diff.resize.Spline36(width=scaled_width * 2, height=height * 2).text.FrameNum(9)
+        diff_clip = diff_clip.resize.Spline36(width=scaled_width * 2, height=height * 2).text.FrameNum(9)
         a, b = (c.resize.Spline36(width=scaled_width, height=height).text.FrameNum(9) for c in (a, b))
 
         diff_stack = Stack({f'{name_a}': core.std.Splice([a[f] for f in frames]),
                             f'{name_b}': core.std.Splice([b[f] for f in frames])}).clip
-        diff = diff.text.Text(text='diff', alignment=8)
-        diff = core.std.Splice([diff[f] for f in frames])
+        diff_clip = diff_clip.text.Text(text='diff', alignment=8)
+        diff_clip = core.std.Splice([diff_clip[f] for f in frames])
 
-        comparison = Stack((diff_stack, diff), direction=Direction.VERTICAL).clip
+        comparison = Stack((diff_stack, diff_clip), direction=Direction.VERTICAL).clip
 
     if return_ranges:
         return comparison, list(_to_ranges(frames))
@@ -708,11 +714,10 @@ def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode
 
     :return:            An interleaved clip of all the `clips`/`namedclips` specified.
 
-    :raises ClipsAndNamedClipsError:
-                        Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("interleave")
+        raise ClipsAndNamedClipsError(interleave)
     return Interleave(clips if clips else namedclips).clip
 
 
@@ -730,11 +735,10 @@ def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     :return:            A clip with the same dimensions as any one of the input clips.
                         with all `clips`/`namedclips` represented as individual vertical slices.
 
-    :raises ClipsAndNamedClipsError:
-                        Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("split")
+        raise ClipsAndNamedClipsError(split)
     return Split(clips if clips else namedclips, label_alignment=2).clip
 
 
@@ -748,11 +752,10 @@ def stack_horizontal(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Vid
 
     :return:            A horizontal stack of the `clips`/`namedclips`.
 
-    :raises ClipsAndNamedClipsError:
-                        Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("stack_horizontal")
+        raise ClipsAndNamedClipsError(stack_horizontal)
     return Stack(clips if clips else namedclips).clip
 
 
@@ -766,11 +769,10 @@ def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Video
 
     :return:            A vertical stack of the `clips`/`namedclips`.
 
-    :raises ClipsAndNamedClipsError:
-                        Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("stack_vertical")
+        raise ClipsAndNamedClipsError(stack_vertical)
     return Stack(clips if clips else namedclips, direction=Direction.VERTICAL).clip
 
 
@@ -789,18 +791,17 @@ def tile(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     :return:            A clip with all input `clips`/`namedclips` automatically tiled most optimally
                         into a rectangular arrrangement.
 
-    :raises ClipsAndNamedClipsError:
-                        Both positional and named clips are given.
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
     """
     if clips and namedclips:
-        raise ClipsAndNamedClipsError("tile")
+        raise ClipsAndNamedClipsError(tile)
     return Tile(clips if clips else namedclips).clip
 
 
 def source_mediainfo(filepath: str, print_mediainfo: bool = False,
                      print_props: bool = False, **source_kwargs: Any) -> vs.VideoNode:
     """
-    Simple source indexing wrapper that adds mediainfo as props as well.
+    Wrap ``source`` and add MediaInfo to the clip as frameprops.
 
     This works exactly like :py:func:`misc.source`,
     except it further gathers information pertaining to the source file.
@@ -820,8 +821,7 @@ def source_mediainfo(filepath: str, print_mediainfo: bool = False,
                                 about the sources you're comping.
     :param source_kwargs:       Keyword arguments passed to :py:func:`misc.source`.
 
-    :raises DependencyNotFoundError:
-                                Dependencies are missing.
+    :raises DependencyNotFoundError: Dependencies are missing.
 
     :return:                    Clip with MediaInfo properties added.
     """
@@ -899,7 +899,7 @@ def source_mediainfo(filepath: str, print_mediainfo: bool = False,
             "bitrate", "vbv_maxrate", "vbv_bufsize", "no-sao-non-deblock", "sao-non-deblock",
             "no-strong-intra-smoothing", "strong-intra-smoothing", "rc-lookahead", "merange", "zones",
             # x264 (incl. dupes, but set will dedupe it)
-            "mbtree", "no-mbtree", "aq", "rc",  "ratetol",
+            "mbtree", "no-mbtree", "aq", "rc", "ratetol",
         }
 
         for x in split_settings:
@@ -952,7 +952,59 @@ def source_mediainfo(filepath: str, print_mediainfo: bool = False,
     return clip
 
 
+def comparison_shots(*clips: vs.VideoNode,
+                     left: int = 0, right: int = 0, top: int = 0, bottom: int = 0,
+                     height: int | None = None, kernel: KernelT = Point,
+                     **namedclips: vs.VideoNode) -> vs.VideoNode:
+    """
+    Convenience function that crops all the given clips, optionally upscales them,
+    and stacks them to allow the user to see multiple filters side-by-side.
+
+    This is intended more so to create comparisons to use in guides, papers,
+    maybe an example to stick in a cheeky comment somewhere. Current name pending™.
+    Alias for this function is ``lvsfunc.cshots``.
+
+    I suggest you get the crops using vspreview's built-in crop helper (found under `misc`).
+    You can then pass the left/right/top/bottom arguments to this function.
+    If the crops result in a very small image, you can upscale it using `height`.
+
+    :param clips:       Clips for comparison (order is kept left to right).
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the top left with their `name`.
+    :param left:        Left crop. Can't be negative.
+    :param right:       Right crop. Can't be negative.
+    :param top:         Top crop. Can't be negative.
+    :param bottom:      Bottom crop. Can't be negative.
+    :param height:      Height to upscale the clips to. If `None`, do not upscale. Default: None.
+    :param kernel:      Kernel used for upscaling the clips if applicable. Default: Point.
+
+    :return:            A horizontal stack of the `clips`/`namedclips`, cropped and upscaled as specified.
+
+    :raises ClipsAndNamedClipsError:    Both positional and named clips are given.
+    """
+    if clips and namedclips:
+        raise ClipsAndNamedClipsError(comparison_shots)
+
+    kernel = Kernel.ensure_obj(kernel, comparison_shots)
+
+    if clips:
+        clips = tuple([c.std.Crop(left, right, top, bottom) for c in clips])
+    elif namedclips:
+        namedclips = {k: v.std.Crop(left, right, top, bottom) for k, v in namedclips.items()}
+
+    if height is None:
+        return Stack(clips if clips else namedclips, direction=Direction.HORIZONTAL).clip
+
+    if clips:
+        clips = tuple([kernel.scale(c, get_w(height), height) for c in clips])
+    elif namedclips:
+        namedclips = {k: kernel.scale(v, get_w(height), height) for k, v in namedclips.items()}
+
+    return Stack(clips if clips else namedclips, direction=Direction.HORIZONTAL).clip
+
+
 # Aliases
 comp = compare
 scomp = stack_compare
 srcm = source_mediainfo
+cshots = comparison_shots
